



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter trust --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
jupyter-trust --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
pip install pep440
IF %ERRORLEVEL% NEQ 0 exit /B 1
pytest -vv --cov nbformat --cov-report term-missing:skip-covered --no-cov-on-fail --cov-fail-under 79
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
